/*MA****************************************************************/
/*                                                                 */
/*  FILE      basic-functions.C                                    */
/*  MODULE    basic-functions                                      */
/*  AUTHOR    Helmut Schmid, IMS, University of Stuttgart          */
/*                                                                 */
/*  PURPOSE   library containing some frequently used functions    */
/*                                                                 */
/*ME****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <assert.h>

#include "basic-functions.h"

#define BUFFER_SIZE 1000


/*FA****************************************************************/
/*                                                                 */
/*  itoa                                                           */
/*                                                                 */
/*FE****************************************************************/

char *itoa( int n )

{
  static char buffer[20];
  sprintf(buffer,"%d",n);
  return &buffer[0];
}


/*FA****************************************************************/
/*                                                                 */
/*  warning                                                        */
/*                                                                 */
/*FE****************************************************************/

void warning( char *message )

{
  fprintf(stderr,"WARNING: %s!\n", message);
}


/*FA****************************************************************/
/*                                                                 */
/*  warning2                                                       */
/*                                                                 */
/*FE****************************************************************/

void warning2( char *message, char *arg )

{
  fprintf(stderr,"WARNING: %s: %s\n", message, arg);
}


/*FA****************************************************************/
/*                                                                 */
/*  error                                                          */
/*                                                                 */
/*FE****************************************************************/

void error( char *message )

{
  fprintf(stderr,"\nERROR: %s!\naborted.\n", message);
  assert(0);
  exit(1);
}


/*FA****************************************************************/
/*                                                                 */
/*  error2                                                         */
/*                                                                 */
/*FE****************************************************************/

void error2( char *message, char *input )

{
  fprintf(stderr,"\nERROR: %s: %s\naborted.\n", message, input);
  assert(0);
  exit(1);
}


/*FA****************************************************************/
/*                                                                 */
/*  error_in_line                                                  */
/*                                                                 */
/*FE****************************************************************/

void error_in_line( char *string, long line )

{
  char buffer[100];

  sprintf(buffer,"%ld", line);
  error2(string, buffer);
}


/*FA****************************************************************/
/*                                                                 */
/*  open_file                                                      */
/*                                                                 */
/*FE****************************************************************/

FILE *open_file( char *filename, char* flags )

{
  FILE *file;

  if ((file = fopen(filename,flags)) == NULL)
    error2("Can't open file",filename);
  return file;
}


/*FA****************************************************************/
/*                                                                 */
/* check_arg                                                       */
/*                                                                 */
/*FE****************************************************************/

void check_arg(void *arg, char *option)

{
  if (arg) return;
  fprintf(stderr,"\nERROR: missing argument to option %s!\n", option);
  exit(1);
}


/*FA****************************************************************/
/*                                                                 */
/* inputs                                                          */
/*                                                                 */
/*FE****************************************************************/

char *inputs( FILE *file )

{
  int  i, c;
  char buffer[BUFFER_SIZE];

  for( i=0; i<BUFFER_SIZE; i++) {
    c = getc(file);
    buffer[i] = (char)c;
    if (c <= 0)
      break;
  }
  if (c != 0)
    error("while reading string from file");

  return( strdup(buffer) );
}


/*FA****************************************************************/
/*                                                                 */
/* outputs                                                         */
/*                                                                 */
/*FE****************************************************************/

void outputs( char *string, FILE *file )

{
  fputs(string, file);
  fputc('\0', file);
}


/*FA****************************************************************/
/*                                                                 */
/* next_string                                                     */
/*                                                                 */
/*FE****************************************************************/

char *next_string( char **start )

{
  char *p, *res;

  /* ignore leading whitespace */
  for( p=*start; isspace((unsigned char)*p); p++) ;

  /* find string end */
  res = p;
  for( ; *p && !isspace((unsigned char)*p); p++) ;

  /* mark end of string */
  if (*p)
    *(p++) = 0;
  *start = p;

  if (*res == 0)
    return NULL;
  else
    return res;
}



/*FA****************************************************************/
/*                                                                 */
/* next_string_with_blanks                                         */
/*                                                                 */
/*FE****************************************************************/

char *next_string_with_blanks( char **start )

{
  char *p, *res;

  /* ignore leading whitespace */
  for( p=*start; isspace((unsigned char)*p); p++) ;

  /* find string end */
  res = p;
  for( ; *p && *p!='\t' && *p!='\n'; p++) /* Blanks are allowed within words */
    ;

  /* mark end of string */
  if (*p) {
    *p = 0;
    *start = p+1;
  }
  else
    *start = p;

  /* remove trailing whitespace */
  for( --p; p>=res && isspace((unsigned char)*p); p--) ;
  *(++p) = 0;

  if (*res == 0)
    return NULL;
  else
    return res;
}


/*FA****************************************************************/
/*                                                                 */
/*  get_line                                                       */
/*                                                                 */
/*FE****************************************************************/

char *get_line( char *buffer, int size, FILE *file )

{
  char *result;

  buffer[size-1] = 1;
  result = fgets(buffer, size, file);
  if (buffer[size-1] == 0)
    error2("Line is too long at", buffer);
  return result;
}

/*FA****************************************************************/
/*                                                                 */
/*  strcat_copy                                                    */
/*                                                                 */
/*FE****************************************************************/

char *strcat_copy( char *string1, char *string2 )

{
  char *result, *p;

  GET_SPACE1(result, char, strlen(string1)+strlen(string2)+1);
  for( p=result; *string1; p++, string1++)
    *p = *string1;
  for( ; *string2; p++, string2++)
    *p = *string2;
  *p = 0;

  return result;
}


/*FA****************************************************************/
/*                                                                 */
/*  quote                                                          */
/*                                                                 */
/*FE****************************************************************/

char *quote( char *string, char qc )

{
  static char buffer[1001];
  int i,k=0;

  if (string[0] == '\'' && string[strlen(string)-1] == '\'')
    return string;
  if (qc)
    buffer[k++] = qc;
  for( i=0; i<1000; i++) {
    if (!string[i])
      break;
    if (string[i] == qc || 
	(qc != '\'' && string[i]=='\\') ||
	(qc=='\0' && string[i]==' '))
      buffer[k++] = '\\';
    buffer[k++] = string[i];
  }
  if (qc)
    buffer[k++] = qc;
  buffer[k] = '\0';

  if (k == 1000)
    warning2("string too large in function \"quote\"", string);

  return &buffer[0];
}


/*FA****************************************************************/
/*                                                                 */
/*  unquote                                                        */
/*                                                                 */
/*FE****************************************************************/

char *unquote(char *string)

{
  char *new_string=string, *s=string;
  char qc = string[0];

  /* Check whether the string is enclosed with quotation marks */
  if (qc == '"')
    string++;  /* skip leading quotation mark */
  else if (qc == '\'')
    *(s++) = *(string++);
  else
    qc = 0;  /* no quotation marks around string */

  /* unquote string */
  for( ;; s++, string++) {
    if (qc != '\'' && *string == '\\')
      string++;    /* undo quotation */
    if ((*s = *string) == '\0') break;
  }

  if (qc) {
    /* check final quotation mark */
    if (*(--s) != qc)
      error2("unterminated string in function \"unquote_string\"", string);
    if (qc == '"')
      *s = '\0'; /* remove quotation */
  }

  return(new_string);
}
