
/*MA****************************************************************/
/*                                                                 */
/*  FILE      basic-functions.h                                    */
/*  MODULE    basic_functions                                      */
/*  AUTHOR    Helmut Schmid, IMS, University of Stuttgart          */
/*                                                                 */
/*  PURPOSE   library containing some frequently used functions    */
/*                                                                 */
/*ME****************************************************************/

#ifndef BASIC_FUNCTIONS_H
#define BASIC_FUNCTIONS_H

#include <stdio.h>
#include <stdlib.h>

#define NIL -1
#define UNDEFINED -2

void warning( char *message );
void warning2( char *message, char *arg );
void error( char *message );
void error2( char *message, char *input );
void error_in_line( char *string, long line );
void check_arg(void *arg, char *option);
FILE *open_file( char *filename, char* flags );
char *next_string( char **start );
char *next_string_with_blanks( char **start );
char *get_line( char *buffer, int size, FILE *file );
char *strcat_copy( char *string1, char *string2 );
char *quote( char *string, char qc );
char *unquote(char *string);
char *itoa( int n );


/*FA****************************************************************/
/*                                                                 */
/*  macros for memory allocation                                   */
/*                                                                 */
/*FE****************************************************************/

#define GET_SPACE( var, type )                       \
{                                                    \
  if ((var = (type*) malloc (sizeof(type))) == NULL) \
    error("Not enough memory");                      \
}

#define GET_SPACE1( var, type, count )                                   \
{                                                                        \
  if ((var = (type*) malloc (sizeof(type) * (unsigned)(count))) == NULL) \
    error("Not enough memory");                                          \
}

#define GET_INIT_SPACE1( var, type, count, value )                       \
{                                                                        \
  int _i;                                                                \
  if ((var = (type*) malloc (sizeof(type) * (unsigned)(count))) == NULL) \
    error("Not enough memory");                                          \
  for( _i=0; _i<(count); _i++)                                           \
    var[_i] = value;                                                     \
}

#define GET_SPACE2( var, type, count1, count2 )                              \
{                                                                            \
  int _i;                                                                    \
								             \
  if ((var = (type**) malloc (sizeof(type*)*(unsigned)(count1))) == NULL)    \
    error("Not enough memory");                                              \
  for( _i=0; _i<(count1); _i++)                                              \
    if ((var[_i] = (type*) malloc (sizeof(type)*(unsigned)(count2))) == NULL)\
      error("Not enough memory");                                            \
}

#define GET_INIT_SPACE2( var, type, count1, count2, value )                  \
{                                                                            \
  int _i,_k;                                                                 \
								             \
  if ((var = (type**) malloc (sizeof(type*)*(unsigned)(count1))) == NULL)    \
    error("Not enough memory");                                              \
  for( _i=0; _i<(count1); _i++)                                              \
    {                                                                        \
      if ((var[_i] = (type*)malloc(sizeof(type)*(unsigned)(count2))) == NULL)\
        error("Not enough memory");                                          \
      for( _k=0; _k<(count2); _k++)                                          \
	var[_i][_k] = value;                                                 \
    }                                                                        \
}

#define FREE2( var, count )                    \
{                                              \
  int _i;                                      \
					       \
  for( _i=0; _i<count; _i++)                   \
    free(var[_i]);                             \
  free(var);                                   \
}


/*FA****************************************************************/
/*                                                                 */
/*  macros for binary file operations                              */
/*                                                                 */
/*FE****************************************************************/

#define OUTPUT( var, file ) \
  if (fwrite( &(var), sizeof var, 1, file) != 1)\
    error("Can't write to output file");

#define OUTPUTS( text, file )    \
  if (fputs( text, file) == EOF) \
    error("while writing to output file");

#define OUTPUT1( var, count, file )                           \
  if (fwrite( var, sizeof *var, count, output_file) != count) \
    error("while writing to output file");

#define INPUT( var, file ) \
  if (fread( &(var), sizeof var, 1, file) != 1) \
    error("while reading from input file");

#define INPUT1( var, count, file ) \
  if (fread( var, sizeof *var, count, file) != count) \
    error("while reading from input file");

char *inputs( FILE *file );

void outputs( char *string, FILE *file );


#endif /* BASIC_FUNCTIONS_H */
