
/*MA****************************************************************/
/*                                                                 */
/*  FILE      hashing.h                                            */
/*  AUTHOR    Helmut Schmid                                        */
/*  PURPOSE   general hashing module with dynamic hash table size  */
/*                                                                 */
/*ME****************************************************************/


typedef unsigned long KEY_TYPE;

typedef struct hash_elem_t
{
  void   *data;
  struct hash_elem_t *next;
} HASH_ELEM;


typedef struct
{
  int  shift_factor;
  long size;
  long number_of_hashes;
  HASH_ELEM **entry;
  KEY_TYPE (*hash_function)();
  int (*identical_data)();
} HASH_TABLE;


/* Interface functions: */

HASH_TABLE *make_hash_table( KEY_TYPE (*compute_key)( void *data ), 
			     int (*compare_items)( void *data1, void *data2 ));
void free_hash_table( HASH_TABLE *hash_table );
void *lookup_data( HASH_TABLE *hash_table, void *item );
int hash_data( HASH_TABLE *hash_table, void *item );
void hash_table_stat( HASH_TABLE *hash_table );
int remove_data( HASH_TABLE *ht, void *data );


/* 
Description:
   
"make_hash_table" creates a new hash table and returns a pointer to
it. The function requires two arguments: The first argument is a
pointer to a funtion which computes a number of type KEY_TYPE from a
given data item. This number is used to compute the hash key. The
second argument is a pointer to a function which compares two data
items and returns 0 if the two are different.

"free_hash_table" frees all the space which has been allocated for the
hash table ht. It does not free the data items themselves.

"lookup_data" checks whether the data item pointed to by the second
argument is stored in the hash table pointed to by the first argument.
The function returns a pointer to the item if it is found and NULL,
otherwise.

"hash_data" stores the data item which the second argument points to
in the hash table which the first argument points to.

"hash_table_stat" prints statistics about how well the hashing worked.

"remove_data" checks whether the data item pointed to by the second
argument is stored in the hash table pointed to by the first argument.
If it is, then it is removed and the return value is 1. Otherwise the 
return value is 0.
*/
