/*MA****************************************************************/
/*                                                                 */
/*  FILE     parse-forest.h                                        */
/*  MODULE   parse-forest                                          */
/*  PROGRAM  vpf                                                   */
/*  AUTHOR   Helmut Schmid, IMS, University of Stuttgart           */
/*                                                                 */
/*ME****************************************************************/

#include <stdio.h>

#define IMPLODED_MASK     1
#define HIGHLIGHTED_MASK  2
#define TAG_MASK          4
#define HIDDEN_MASK       8

typedef enum {terminal, tree, disjunct, fs, shared, column, row} NODE_TYPE;

typedef struct {
  unsigned short x, y;
} XY;

typedef struct {
  int tag_number;
  XY  tag_size;
  struct node_t *subnode;
} SHARED;

typedef struct {
  int number_of_daughters;
  struct node_t **daughter;
} NTNODE;

typedef struct {
  char *name;
  int  label_width;
  int  padding;
  struct node_t *subnode;
} TNODE;

typedef struct {
  NODE_TYPE type;
  union {
    TNODE  tnode;
    NTNODE node;
    SHARED shared;
  } data;
} VALUE;

typedef struct node_t {
  unsigned short flags;
  VALUE value;
  XY pos;
  XY size;
} NODE;

typedef struct node_list {
  NODE *node;
  struct node_list *next;
} NODE_LIST;


NODE_LIST *append( NODE *node, NODE_LIST *nlist );
NODE *new_node( NODE_LIST *nlist, NODE_TYPE type );
NODE *new_tnode( char *string, NODE *node);
NODE *new_shared_node( int tag, NODE *node );
void print_parses( FILE *file, int selected );
void parse_forest_init( void );
void insert_node_references( void );

extern NODE *AllParses;
