%{
/*MA****************************************************************/
/*                                                                 */
/*  FILE     reader.y                                              */
/*  MODULE   reader                                                */
/*  PROGRAM  vpf                                                   */
/*  AUTHOR   Helmut Schmid, IMS, University of Stuttgart           */
/*                                                                 */
/*ME****************************************************************/

#include <assert.h>

#include "basic-functions.h"
#include "parse-forest.h"

extern int  yylineno;
extern char *yytext;

void yyerror(char *text);
int yylex( void );
%}

%union {
  int       number;
  char      *string;
  NODE      *node;
  NODE_LIST *nodelist;
}

%token <number>    DFLAG RFLAG IFLAG HFLAG RM CM
%token <string>    STRING

%type  <node>      N
%type  <nodelist>  NLIST

%%

ALL:   NLIST           { AllParses=new_node($1, disjunct); }
     ;

NLIST: N NLIST         { $$=append($1,$2); }
     | /* empty */     { $$=NULL; }
     ;

N:     IFLAG N         { $$=$2; $$->flags |= IMPLODED_MASK; }
     | HFLAG N         { $$=$2; $$->flags |= HIDDEN_MASK; }
     | DFLAG N         { $$=new_shared_node($1,$2); }
     | RFLAG           { $$=new_shared_node($1, NULL); }
     | '(' N NLIST ')' { $$=new_node(append($2,$3), tree); }
     | '[' NLIST ']'   { $$=new_node($2, fs); }
     | '{' NLIST '}'   { $$=new_node($2, disjunct); }
     | '<' NLIST '>'   { $$=new_node($2, column); }
     | '^' NLIST '$'   { $$=new_node($2, row); }
     | STRING '=' N    { $$=new_tnode($1,$3); }
     | STRING          { $$=new_tnode($1,NULL); }
     | '(' STRING ')'  { $$=new_tnode($2,NULL); }
     ;

%%


extern FILE  *yyin;


/*FA****************************************************************/
/*                                                                 */
/*  yyerror                                                        */
/*                                                                 */
/*FE****************************************************************/

void yyerror(char *text)

{
  error2(text,yytext);
}



/*FA****************************************************************/
/*                                                                 */
/*  read_parses                                                    */
/*                                                                 */
/*FE****************************************************************/

void read_parses( FILE *file )

{
  yyin = file;
  parse_forest_init();
  yyparse();
  insert_node_references();
}


#ifdef TEST
/*FA****************************************************************/
/*                                                                 */
/*  print_usage                                                    */
/*                                                                 */
/*FE****************************************************************/

void print_usage( char *prog )

{
  fprintf(stderr,"\nUsage: %s [ file ]\n", prog);
}


/*FA****************************************************************/
/*                                                                 */
/*  main                                                           */
/*                                                                 */
/*FE****************************************************************/

int main( int argc, char *argv[] )

{
  FILE *file;

  if (argc == 2) {
    if (strcmp(argv[1], "-help") == 0 || strcmp(argv[1], "-h") == 0) {
      print_usage(argv[0]);
      exit(0);
    }
    if ((file = fopen(argv[1], "rt")) == NULL)
      error2("Can't open file", argv[1]);
  }
  else if (argc == 1)
    file = stdin;
  else {
    print_usage(argv[0]);
    exit(1);
  }

  read_parses( file );
  print_parses( stdout );

  return 0;
}
#endif
