/*MA****************************************************************/
/*                                                                 */
/*  FILE     tag.c                                                 */
/*  MODULE   tag                                                   */
/*  PROGRAM  vpf                                                   */
/*  AUTHOR   Helmut Schmid, IMS, University of Stuttgart           */
/*  PURPOSE  routines dealing with reentrancies                    */
/*                                                                 */
/*ME****************************************************************/

#include <stdio.h>
#include <assert.h>

#include "hashing.h"
#include "basic-functions.h"
#include "parse-forest.h"
#include "tag.h"

typedef struct {
  int  number;
  NODE *node;
  NODE *parent;
} TAG;


/* Two hash tables are used. The first retrieves the node for */
/* a given tag; the second retrieves the tag for a given node */

HASH_TABLE *HT;
HASH_TABLE *HT2;


/*FA****************************************************************/
/*                                                                 */
/*  hashf                                                          */
/*                                                                 */
/*FE****************************************************************/

static KEY_TYPE hashf( void *data )

{
  TAG *tag=(TAG*)data;

  return (KEY_TYPE)tag->number;
}


/*FA****************************************************************/
/*                                                                 */
/*  hashf2                                                         */
/*                                                                 */
/*FE****************************************************************/

static KEY_TYPE hashf2( void *data )

{
  TAG *tag=(TAG*)data;

  return (KEY_TYPE)tag->node;
}


/*FA****************************************************************/
/*                                                                 */
/*  identical_data                                                 */
/*                                                                 */
/*FE****************************************************************/

static int identical_data( void *data1, void *data2 )

{
  TAG *tag1=(TAG*)data1;
  TAG *tag2=(TAG*)data2;

  return (tag1->number == tag2->number);
}


/*FA****************************************************************/
/*                                                                 */
/*  identical_data2                                                */
/*                                                                 */
/*FE****************************************************************/

static int identical_data2( void *data1, void *data2 )

{
  TAG *tag1=(TAG*)data1;
  TAG *tag2=(TAG*)data2;

  return (tag1->node == tag2->node);
}


/*FA****************************************************************/
/*                                                                 */
/*  tag_init                                                       */
/*                                                                 */
/*FE****************************************************************/

void tag_init()

{
  HT = make_hash_table(hashf, identical_data);
  HT2 = make_hash_table(hashf2, identical_data2);
}


/*FA****************************************************************/
/*                                                                 */
/*  get_node_by_tag                                                */
/*                                                                 */
/*FE****************************************************************/

NODE *get_node_by_tag( int number )

{
  TAG tag, *p;

  tag.number = number;
  p = (TAG*)lookup_data(HT, (void*)&tag);
  if (p)
    return p->node;
  else
    return NULL;
}


/*FA****************************************************************/
/*                                                                 */
/*  insert_tagged_node                                             */
/*                                                                 */
/*FE****************************************************************/

void insert_tagged_node( int number, NODE *node, NODE *parent )

{
  TAG *tag;

  GET_SPACE(tag, TAG);
  tag->number = number;
  tag->node = node;
  tag->parent = parent;
  hash_data( HT, (void*)tag );
  hash_data( HT2, (void*)tag );
}


/*FA****************************************************************/
/*                                                                 */
/*  set_selected                                                   */
/*                                                                 */
/*FE****************************************************************/

void set_selected( NODE *node )

{
  TAG tag, *p;

  tag.node = node->value.data.shared.subnode;
  p = (TAG*)lookup_data(HT2, (void*)&tag);
  assert(p);
  p->parent->flags |= TAG_MASK;
  node->flags ^= TAG_MASK;

  p->parent = node;
}


/*FA****************************************************************/
/*                                                                 */
/*  is_selected                                                    */
/*                                                                 */
/*FE****************************************************************/

int is_selected( NODE *node )

{
  TAG tag, *p;

  tag.node = node->value.data.shared.subnode;
  p = (TAG*)lookup_data(HT2, (void*)&tag);
  assert(p);

  return (p->parent == node);
}


/*FA****************************************************************/
/*                                                                 */
/*  tag_number                                                     */
/*                                                                 */
/*FE****************************************************************/

int tag_number( NODE *node )

{
  TAG tag, *p;

  tag.node = node->value.data.shared.subnode;
  p = (TAG*)lookup_data(HT2, (void*)&tag);
  assert(p);

  return p->number;
}
