/*MA****************************************************************/
/*                                                                 */
/*  FILE     tcl-interface.c                                       */
/*  MODULE   tcl-interface                                         */
/*  PROGRAM  vpf                                                   */
/*  AUTHOR   Helmut Schmid, IMS, University of Stuttgart           */
/*  PURPOSE  wrapper functions for the basic C routines            */
/*                                                                 */
/*ME****************************************************************/

#include <stdlib.h>
#include <tcl.h>
#include <assert.h>

#include "parse-forest.h"

#include "basic-functions.h"
#include "draw-functions.h"
#include "tcl-interface.h"

Tcl_Interp *Interp;


/*FA****************************************************************/
/*                                                                 */
/*  Vpf_ParseForestCount                                           */
/*                                                                 */
/*FE****************************************************************/

static int Vpf_ParseForestCount( ClientData clientData, Tcl_Interp *interp, 
				 int argc, char *argv[] )
{
  Tcl_SetResult(interp, itoa(parse_forest_count()), TCL_VOLATILE);
  return TCL_OK;
}


/*FA****************************************************************/
/*                                                                 */
/*  Vpf_GetDrawCommands                                            */
/*                                                                 */
/*FE****************************************************************/

static int Vpf_GetDrawCommands( ClientData clientData, Tcl_Interp *interp, 
				int argc, char *argv[] )

{
  char *command_string;

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  if (Interp != interp)
    Interp = interp;
  command_string = draw_commands( atoi(argv[1]) - 1 );
  if (command_string == NULL) {
    Tcl_SetResult(interp, "error in draw function", TCL_STATIC);
    return TCL_ERROR;
  }

  Tcl_SetResult(interp, command_string, TCL_VOLATILE);
  return TCL_OK;
}


/*FA****************************************************************/
/*                                                                 */
/*  Vpf_Highlight                                                  */
/*                                                                 */
/*FE****************************************************************/

static int Vpf_Highlight( ClientData clientData, Tcl_Interp *interp, 
			  int argc, char *argv[] )

{
  if (argc != 3) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  if (set_highlighted( atoi(argv[1]), atoi(argv[2]) ))
    Tcl_SetResult(interp, "redisplay", TCL_STATIC);
  else
    Tcl_SetResult(interp, "", TCL_STATIC);

  return TCL_OK;
}


/*FA****************************************************************/
/*                                                                 */
/*  Vpf_Save                                                       */
/*                                                                 */
/*FE****************************************************************/

static int Vpf_Save( ClientData clientData, Tcl_Interp *interp, 
		     int argc, char *argv[] )

{
  FILE *file;

  if (argc != 3) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  if ((file = fopen(argv[1], "wt")) == NULL)
    return TCL_ERROR;
  print_parses(file, atoi(argv[2]));
  fclose(file);

  return TCL_OK;
}


/*FA****************************************************************/
/*                                                                 */
/*  Vpf_Implode                                                    */
/*                                                                 */
/*FE****************************************************************/

static int Vpf_Implode( ClientData clientData, Tcl_Interp *interp, 
			int argc, char *argv[] )

{
  if (argc != 3) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  toggle_imploded( atoi(argv[1]), atoi(argv[2]) );
  return TCL_OK;
}


/*FA****************************************************************/
/*                                                                 */
/*  Vpf_Init                                                       */
/*                                                                 */
/*FE****************************************************************/

int Vpf_Init( Tcl_Interp *interp )

{
  Interp = interp;
  
  Tcl_CreateCommand( interp,
		     "parseForestCount",
		     Vpf_ParseForestCount,
		     (ClientData) NULL,
		     (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand( interp,
		     "drawCommands", 
		     Vpf_GetDrawCommands,
		     (ClientData) NULL,
		     (Tcl_CmdDeleteProc *) NULL);
  
  Tcl_CreateCommand( interp,
		     "highlight", 
		     Vpf_Highlight,
		     (ClientData) NULL,
		     (Tcl_CmdDeleteProc *) NULL);
  
  Tcl_CreateCommand( interp,
		     "implode",
		     Vpf_Implode,
		     (ClientData) NULL,
		     (Tcl_CmdDeleteProc *) NULL);
  
  Tcl_CreateCommand( interp,
		     "save",
		     Vpf_Save,
		     (ClientData) NULL,
		     (Tcl_CmdDeleteProc *) NULL);
  return 1;
}
