#MA################################################################
#                                                                 #
#  FILE     vpf-init.tcl                                          #
#  PROGRAM  vpf                                                   #
#  AUTHOR   Helmut Schmid, IMS, University of Stuttgart           #
#  PURPOSE  implements the Tcl user interface of vpf              #
#                                                                 #
#ME################################################################

option add *Canvas.foreground blue

#################################################################
#                Tooltip Functions                              #
#################################################################

option add *Balloonhelp*background lightblue userDefault
option add *Balloonhelp*foreground black userDefault
option add *Balloonhelp.info.wrapLength 3i userDefault
option add *Balloonhelp.info.justify left userDefault
option add *Balloonhelp.info.font \
    -*-lucida-medium-r-normal-sans-*-120-* userDefault

toplevel .balloonhelp -class Balloonhelp -borderwidth 1 -relief flat

label .balloonhelp.info
pack .balloonhelp.info -side left -fill y

wm overrideredirect .balloonhelp 1
wm withdraw .balloonhelp

# ----------------------------------------------------------------------
#  USAGE:  balloonhelp_for <win> <mesg>
#
#  Adds balloon help to the widget named <win>.  Whenever the mouse
#  pointer enters this widget and rests within it for a short delay,
#  a balloon help window will automatically appear showing the
#  help <mesg>.
# ----------------------------------------------------------------------
proc balloonhelp_for {win mesg} {
    global bhInfo
    set bhInfo($win) $mesg

    bind $win <Enter> {balloonhelp_pending %W}
    bind $win <Leave> {balloonhelp_cancel}
}

# ----------------------------------------------------------------------
#  USAGE:  balloonhelp_control <state>
#
#  Turns balloon help on/off for the entire application.
# ----------------------------------------------------------------------
set bhInfo(active) 1

proc balloonhelp_control {state} {
    global bhInfo

    if {$state} {
        set bhInfo(active) 1
    } else {
        balloonhelp_cancel
        set bhInfo(active) 0
    }
}

# ----------------------------------------------------------------------
#  USAGE:  balloonhelp_pending <win>
#
#  Used internally to mark the point in time when a widget is first
#  touched.  Sets up an "after" event so that balloon help will appear
#  if the mouse remains within the current window.
# ----------------------------------------------------------------------
proc balloonhelp_pending {win} {
    global bhInfo

    balloonhelp_cancel
    set bhInfo(pending) [after 1500 [list balloonhelp_show $win]]
}

# ----------------------------------------------------------------------
#  USAGE:  balloonhelp_cancel
#
#  Used internally to mark the point in time when the mouse pointer
#  leaves a widget.  Cancels any pending balloon help requested earlier
#  and hides the balloon help window, in case it is visible.
# ----------------------------------------------------------------------
proc balloonhelp_cancel {} {
    global bhInfo

    if {[info exists bhInfo(pending)]} {
        after cancel $bhInfo(pending)
        unset bhInfo(pending)
    }
    wm withdraw .balloonhelp
}

# ----------------------------------------------------------------------
#  USAGE:  balloonhelp_show <win>
#
#  Used internally to display the balloon help window for the
#  specified <win>.
# ----------------------------------------------------------------------
proc balloonhelp_show {win} {
    global bhInfo

    if {$bhInfo(active)} {
        .balloonhelp.info configure -text $bhInfo($win)

        set x [expr [winfo rootx $win]+10]
        set y [expr [winfo rooty $win]+[winfo height $win]]
        wm geometry .balloonhelp +$x+$y
        wm deiconify .balloonhelp
        raise .balloonhelp
    }
    unset bhInfo(pending)
}


#MA################################################################
#                                                                 #
#  vpf-specific code starts here                                  #
#                                                                 #
#ME################################################################

# change the following line if you need a different font
# option add *Display.font *-times-medium-r-normal-*-12-* userDefault
# use the following line instead for Greek characters
option add *Display.font -greek-helvetica-*-r-* userDefault

set Highlighting 0
set CanvasBackground \#fff8ea
set HighlightColor \#eaeaea
#lightgrey
set index 1
set last [ parseForestCount ]


#MA################################################################
#                                                                 #
#  draw function implementations                                  #
#                                                                 #
#ME################################################################

set SquareBracketWidth 5
set CurlyBracketWidth 10

proc d {x y} {
    .d.c configure -scrollregion "0 0 $x $y"
}

proc h {x1 y1 x2 y2} {
    global HighlightColor Highlighting
    .d.c create rect $x1 $y1 $x2 $y2 -fill $HighlightColor -outline {}
}

proc e {x1 y1 x2 y2} {
    .d.c create oval [expr $x1-1] [expr $y1-1] [expr $x1+1] [expr $y1+1] \
	-fill black
    .d.c create line $x1 $y1 $x2 $y2
    .d.c create oval [expr $x2-1] [expr $y2-1] [expr $x2+1] [expr $y2+1] \
	-fill black
}

proc l {x y t} {
    global font
    # uncomment the following line for Greek characters
    # set t [encoding convertfrom iso8859-7 $t]
    .d.c create text $x $y -text $t -anchor nw -font $font
}

proc i {x1 y1 x2 y2} {
    global ImpWidth
    .d.c create rect $x1 $y1 $x2 $y2
    .d.c create line $x1 $y1 $x2 $y2
    .d.c create line $x1 $y2 $x2 $y1
}

proc t {x1 y1 x2 y2 n} {
    global font
    .d.c create rect [expr $x1-1] $y1 [expr $x2+1] [expr $y2+1] \
	-fill black
    .d.c create text $x1 $y1 -text $n -anchor nw -font $font -fill white
}

proc tt {x1 y1 x2 y2 n} {
    global font
    .d.c create rect [expr $x1-1] $y1 [expr $x2+1] [expr $y2+1]
    .d.c create text $x1 $y1 -text $n -anchor nw -font $font
}

proc lsb {x y len} {
    global SquareBracketWidth
    .d.c create line [expr $x+$SquareBracketWidth] $y \
	$x $y \
	$x [expr $y+$len] \
	[expr $x+$SquareBracketWidth] [expr $y+$len]
}

proc rsb {x y len} {
    global SquareBracketWidth
    .d.c create line [expr $x-$SquareBracketWidth] $y \
	$x $y \
	$x [expr $y+$len] \
	[expr $x-$SquareBracketWidth] [expr $y+$len]
}

proc rsb {x y len} {
    global SquareBracketWidth
    .d.c create line [expr $x-$SquareBracketWidth] $y \
	$x $y \
	$x [expr $y+$len] \
	[expr $x-$SquareBracketWidth] [expr $y+$len]
}

proc lcb {x y len} {
    global CurlyBracketWidth
    .d.c create line [expr $x+$CurlyBracketWidth] $y \
	[expr $x+$CurlyBracketWidth*3/4] $y \
	[expr $x+$CurlyBracketWidth/2] [expr $y+$CurlyBracketWidth/4] \
	[expr $x+$CurlyBracketWidth/2] [expr $y+$len/2-$CurlyBracketWidth/4] \
	[expr $x+$CurlyBracketWidth/4] [expr $y+$len/2-$CurlyBracketWidth/12] \
	$x [expr $y+$len/2] \
	[expr $x+$CurlyBracketWidth/4] [expr $y+$len/2+$CurlyBracketWidth/12] \
	[expr $x+$CurlyBracketWidth/2] [expr $y+$len/2+$CurlyBracketWidth/4] \
	[expr $x+$CurlyBracketWidth/2] [expr $y+$len-$CurlyBracketWidth/4] \
	[expr $x+$CurlyBracketWidth*3/4] [expr $y+$len] \
	[expr $x+$CurlyBracketWidth] [expr $y+$len]
}

proc rcb {x y len} {
    global CurlyBracketWidth
    .d.c create line [expr $x-$CurlyBracketWidth] $y \
	[expr $x-$CurlyBracketWidth*3/4] $y \
	[expr $x-$CurlyBracketWidth/2] [expr $y+$CurlyBracketWidth/4] \
	[expr $x-$CurlyBracketWidth/2] [expr $y+$len/2-$CurlyBracketWidth/4] \
	[expr $x-$CurlyBracketWidth/4] [expr $y+$len/2-$CurlyBracketWidth/12] \
	$x [expr $y+$len/2] \
	[expr $x-$CurlyBracketWidth/4] [expr $y+$len/2+$CurlyBracketWidth/12] \
	[expr $x-$CurlyBracketWidth/2] [expr $y+$len/2+$CurlyBracketWidth/4] \
	[expr $x-$CurlyBracketWidth/2] [expr $y+$len-$CurlyBracketWidth/4] \
	[expr $x-$CurlyBracketWidth*3/4] [expr $y+$len] \
	[expr $x-$CurlyBracketWidth] [expr $y+$len]
}


#MA################################################################
#                                                                 #
#  GUI functions                                                  #
#                                                                 #
#ME################################################################

proc display {} {
    global last index

    if {$index > $last} {
	set index $last
    }
    if {$index < 1} {
	set index 1
    }

    # enable/disable buttons
    if {$index == 1} { .mbar.backward configure -state disabled }
    if {$index >  1} { .mbar.backward configure -state normal }
    if {$index == $last} { .mbar.forward configure -state disabled }
    if {$index <  $last} { .mbar.forward configure -state normal }

    .d.c delete all
    eval [ drawCommands $index ]
}

proc next_sentence {} {
    global index last
    if {$index < $last} {
	incr index
	display
    }
}

proc prev_sentence {} {
    global index
    if {$index > 1} {
	incr index -1
	display
    }
}

proc save_cmd {} {
    toplevel .dialog

    frame .dialog.info
    pack .dialog.info -expand yes -fill both -padx 10 -pady 10
    label .dialog.info.text -text "Please enter the file name"
    pack .dialog.info.text

    set name "output.pf"
    entry .dialog.fileName -width 20 -relief sunken -bd 2 -textvariable name -justify right -background white

    checkbutton .dialog.selected -text "print selected" -variable selected
    pack .dialog.fileName .dialog.selected -padx 5 -pady 5

    balloonhelp_for .dialog.selected "Print only unimploded nodes in case of disjunctions."

    frame .dialog.controls
    pack .dialog.controls -fill x -padx 10 -pady 10

    wm title .dialog "File Selection"
    wm group .dialog .

    button .dialog.controls.ok -text OK -command { 
	if {$name != ""} {
	    save $name $selected
	    grab release .dialog
	    destroy .dialog 
	}
    }
    button .dialog.controls.cancel -text Cancel -command { 
	grab release .dialog
	destroy .dialog
    }

    pack .dialog.controls.ok .dialog.controls.cancel -expand 1 -side left -padx 2 -pady 2

    focus .dialog.fileName
    grab set .dialog
}

proc highlight_cmd {x y} {
    global Highlighting
    set cx [ .d.c canvasx $x ]
    set cy [ .d.c canvasy $y ]
    if {$Highlighting == 1} {
	set result [ highlight $cx $cy ]
	if {$result == "redisplay"} { display }
    }
}

proc implode_cmd {x y} {
    set cx [ .d.c canvasx $x ]
    set cy [ .d.c canvasy $y ]
    implode $cx $cy
    display
}


#MA################################################################
#                                                                 #
#  creation of widgets                                            #
#                                                                 #
#ME################################################################

wm title . "vpf"

frame .mbar -bd 2 -relief groove
button .mbar.backward -text "<" -command prev_sentence
button .mbar.forward -text ">"  -command next_sentence
label .mbar.sentence -text "sentence"
entry .mbar.number -justify right -background $CanvasBackground -width 5 -textvariable index
button .mbar.quit -text "Quit" -command exit
button .mbar.save -text "Save" -command save_cmd
checkbutton .mbar.hilite -text "highlighting" -variable Highlighting

bind .mbar.number <KeyPress-Return> { display }
bind .mbar.number <KeyPress-q> { exit 0 }
bind .mbar.number <Control-q> { exit 0 }

pack .mbar.quit .mbar.save -side right -padx 3 -pady 3
pack .mbar.backward .mbar.forward -side left -padx 3 -pady 3
pack .mbar.hilite -side right -expand 1 -padx 3 -pady 3
pack .mbar.number -side right -padx 3 -pady 3
pack .mbar.sentence -side left -padx 3 -pady 3

balloonhelp_for .mbar.forward "next sentence"
balloonhelp_for .mbar.backward "previous sentence"
balloonhelp_for .mbar.quit "exit vpf"
balloonhelp_for .mbar.hilite "switches highlighting of structures on and off"
balloonhelp_for .mbar.sentence "number of the currently displayed sentence"

frame .d -class Display
canvas .d.c -background $CanvasBackground -height 500 -width 700 \
    -xscrollcommand ".d.xscroll set" -yscrollcommand ".d.yscroll set"
   
scrollbar .d.xscroll -orient horizontal -command ".d.c xview"
scrollbar .d.yscroll -orient vertical -command ".d.c yview"

grid .d.c -row 0 -column 0 -sticky news
grid .d.yscroll -row 0 -column 1 -sticky ns
grid .d.xscroll -row 1 -column 0 -sticky ew

grid columnconfigure .d 0 -weight 1
grid rowconfigure .d 0 -weight 1

pack .mbar -fill x
pack .d -expand yes -fill both


set font [ option get .d font Display ]

bind .d.c <2> { prev_sentence }
bind .d.c <1> { implode_cmd %x %y }
bind .d.c <3> { next_sentence }
bind .d.c <KeyPress-q> { exit 0 }
bind .d.c <Control-q> { exit 0 }
bind .d.c <KeyPress-p> { prev_sentence }
bind .d.c <KeyPress-n> { next_sentence }
bind .d.c <Motion> { 
    balloonhelp_cancel
    highlight_cmd %x %y
    balloonhelp_pending %W
}

focus .d.c


balloonhelp_for .d.c "click middle button to get previous sentence, right button to get next sentence and left button to implode/explode structures"

display
