/*MA****************************************************************/
/*                                                                 */
/*  FILE     vpf.c                                                 */
/*  MODULE   vpf                                                   */
/*  PROGRAM  vpf                                                   */
/*  AUTHOR   Helmut Schmid, IMS, University of Stuttgart           */
/*  PURPOSE                                                        */
/*                                                                 */
/*ME****************************************************************/


#include <stdio.h>
#include <tcl.h>
#include <tk.h>
#include <locale.h>
#include <string.h>

#include "parse-forest.h"
#include "basic-functions.h"
#include "reader.h"
#include "tcl-interface.h"

#ifndef TCL_SCRIPT
#define TCL_SCRIPT "/usr/share/vpf/vpf-init.tcl"
#endif

static char *Tcl_Script = TCL_SCRIPT;


/* The following variable is a special hack that is needed in order for */
/* Sun shared libraries to be used for Tcl.                             */

extern int matherr();
int *tclDummyMathPtr = (int *) matherr;

#ifdef TK_TEST
extern int Tcltest_Init _ANSI_ARGS_((Tcl_Interp *interp));
extern int Tktest_Init _ANSI_ARGS_((Tcl_Interp *interp));
#endif /* TK_TEST */

static int Verbose=1;


/*FA****************************************************************/
/*                                                                 */
/*  Tcl_AppInit                                                    */
/*                                                                 */
/*FE****************************************************************/

int Tcl_AppInit( Tcl_Interp *interp )

{
  /* Create a Tcl interpreter */
  if (Tcl_Init(interp) == TCL_ERROR) { return TCL_ERROR; }
  if (Tk_Init(interp) == TCL_ERROR) { return TCL_ERROR; }

  /* announce that Tk has been linked statically */
  Tcl_StaticPackage(interp, "Tk", Tk_Init, Tk_SafeInit);

#ifdef TK_TEST
  if (Tcltest_Init(interp) == TCL_ERROR) { return TCL_ERROR; }
  Tcl_StaticPackage(interp, "Tcltest", Tcltest_Init,
		    (Tcl_PackageInitProc *) NULL);
  if (Tktest_Init(interp) == TCL_ERROR) { return TCL_ERROR; }
  Tcl_StaticPackage(interp, "Tktest", Tktest_Init,
		    (Tcl_PackageInitProc *) NULL);
#endif /* TK_TEST */

  /*
   * Call the init procedures for included packages.  Each call should
   * look like this:
   * if (Mod_Init(interp) == TCL_ERROR) { return TCL_ERROR; }
   * where "Mod" is the name of the module.
   */
  
  /*
   * Call Tcl_CreateCommand for application-specific commands, if
   * they weren't already created by the init procedures called above.
   */

  Vpf_Init( interp );

  /*
   * Specify a user-specific startup file to invoke if the application
   * is run interactively.  Typically the startup file is "~/.apprc"
   * where "app" is the name of the application.  If this line is deleted
   * then no user-specific startup file will be run under any conditions.
   */
  Tcl_SetVar(interp, "tcl_rcFileName", "~/.vpfrc", TCL_GLOBAL_ONLY);
  
  return TCL_OK;
}


/*FA****************************************************************/
/*                                                                 */
/*  print_usage                                                    */
/*                                                                 */
/*FE****************************************************************/

void print_usage( char *prog )

{
  fprintf(stderr,"\nUsage: %s [ -options- ][ file ]\n\n", prog);
  fputs("OPTIONS:\n",stderr);
  fputs("-f <file>: Read the tcl script from <file> rather than from "TCL_SCRIPT"\n", stderr);
  fputs("-q: quiet mode\n\n", stderr);
  exit(0);
}


/*FA****************************************************************/
/*                                                                 */
/*  get_arg                                                        */
/*                                                                 */
/*  Purpose: processing of command line options and arguments      */
/*                                                                 */
/*FE****************************************************************/

static int get_arg(int *argc, char **argv)

{
  char *opt, *arg;
  int  i, k, d=0;

  for(i=1; i<*argc;) {
    opt = argv[i]; 
    if (i < *argc-1)
      arg = argv[i+1];
    else
      arg = NULL;
    if (opt[0] == '-') {
      if (strcmp(opt,"-f") == 0) {
	if (arg == NULL)
	  error("Missing argument to option -f");
	Tcl_Script = arg;
	d = 2;
      }
      else if (strcmp(opt,"-q") == 0) {
	Verbose = 0;
	d = 1;
      }
      else 
	print_usage(argv[0]);
      *argc -= d;
      for( k=i; k<*argc; k++)
	argv[k] = argv[k+d];
    }
    else
      i++;
  }
  return (*argc <= 2);
}


/*FA****************************************************************/
/*                                                                 */
/*  main                                                           */
/*                                                                 */
/*FE****************************************************************/

int main( int argc, char **argv )

{
  FILE *file;
  char *dummy[2];

  if (!get_arg(&argc,argv)) {
    print_usage(argv[0]);
    exit(1);
  }

  if (argc == 1)
    file = stdin;
  else if ((file = fopen(argv[1], "rt")) == NULL)
    error2("Can't open", argv[1]);

  if (Verbose)
    fprintf(stderr,"reading parse forests..."); 
  read_parses(file);
  if (Verbose)
    fprintf(stderr,"done\n");

  if (AllParses->value.data.node.number_of_daughters == 0)
    error("empty input");

  dummy[0] = argv[0];
  dummy[1] = Tcl_Script;
  Tk_Main(2, dummy, Tcl_AppInit);

  return 0;
}
